@@ -9,7 +9,8 @@ use chrono::prelude::*;
 use chrono::DateTime;
 use lazy_static::lazy_static;
 use serde::{Deserialize, Serialize};
-use serde_json::{Result, Value};
+use serde_json::Value;
+use simple_error::SimpleError;
 use std::collections::{BTreeMap, HashMap};
 
 const EXCHANGE_NAME: &str = "bitmex";
@@ -31,6 +32,7 @@ lazy_static! {
             ("ADAH19", (323, 0.00000001)),
             ("ADAH20", (366, 0.00000001)),
             ("ADAH21", (459, 0.00000001)),
+            ("ADAH22", (761, 0.00000001)),
             ("ADAM18", (271, 0.00000001)),
             ("ADAM19", (331, 0.00000001)),
             ("ADAM20", (383, 0.00000001)),
@@ -40,15 +42,17 @@ lazy_static! {
             ("ADAU20", (398, 0.00000001)),
             ("ADAU21", (570, 0.00000001)),
             ("ADAUSD", (676, 0.0001)),
-            ("ADAUSDT", (521, 0.00001)),
+            ("ADAUSDT", (521, 0.0001)),
             ("ADAUSDTH21", (463, 0.00001)),
             ("ADAUSDTZ20", (431, 0.00001)),
             ("ADAZ18", (303, 0.00000001)),
             ("ADAZ19", (349, 0.00000001)),
             ("ADAZ20", (427, 0.00000001)),
             ("ADAZ21", (638, 0.00000001)),
+            ("ALTMEXTUSDT", (801, 0.01)),
             ("ALTMEXUSD", (616, 0.01)),
             ("AVAXUSD", (675, 0.001)),
+            ("AVAXUSDT", (808, 0.001)),
             ("AXSUSD", (706, 0.01)),
             ("AXSUSDT", (625, 0.001)),
             ("BCHF18", (256, 0.0001)),
@@ -65,6 +69,7 @@ lazy_static! {
             ("BCHU20", (397, 0.00001)),
             ("BCHU21", (569, 0.000001)),
             ("BCHUSD", (402, 0.05)),
+            ("BCHUSDT", (737, 0.05)),
             ("BCHX17", (239, 0.0001)),
             ("BCHZ17", (240, 0.0001)),
             ("BCHZ18", (302, 0.0001)),
@@ -93,11 +98,12 @@ lazy_static! {
             ("DASHM17", (186, 0.000001)),
             ("DASHU17", (209, 0.000001)),
             ("DASHZ17", (225, 0.000001)),
+            ("DEFIMEXTUSDT", (802, 0.01)),
             ("DEFIMEXUSD", (617, 0.01)),
             ("DOGEUSD", (677, 0.00001)),
             ("DOGEUSDT", (476, 0.00001)),
             ("DOTUSD", (679, 0.001)),
-            ("DOTUSDT", (519, 0.0005)),
+            ("DOTUSDT", (519, 0.001)),
             ("DOTUSDTH21", (469, 0.0005)),
             ("DOTUSDTZ20", (453, 0.0005)),
             ("EOSH19", (324, 0.0000001)),
@@ -113,7 +119,7 @@ lazy_static! {
             ("EOSU20", (399, 0.0000001)),
             ("EOSU21", (571, 0.00000001)),
             ("EOSUSD", (707, 0.0001)),
-            ("EOSUSDT", (539, 0.0005)),
+            ("EOSUSDT", (539, 0.0001)),
             ("EOSUSDTH21", (464, 0.0005)),
             ("EOSUSDTZ20", (432, 0.0005)),
             ("EOSZ18", (304, 0.0000001)),
@@ -127,6 +133,7 @@ lazy_static! {
             ("ETHH19", (319, 0.00001)),
             ("ETHH20", (362, 0.00001)),
             ("ETHH21", (455, 0.00001)),
+            ("ETHH22", (759, 0.00001)),
             ("ETHJ17", (177, 0.00001)),
             ("ETHM17", (185, 0.00001)),
             ("ETHM18", (273, 0.00001)),
@@ -140,8 +147,12 @@ lazy_static! {
             ("ETHU21", (566, 0.00001)),
             ("ETHUSD", (297, 0.05)),
             ("ETHUSDH21", (462, 0.05)),
+            ("ETHUSDH22", (762, 0.05)),
             ("ETHUSDM20", (386, 0.05)),
             ("ETHUSDM21", (527, 0.05)),
+            ("ETHUSDT", (734, 0.05)),
+            ("ETHUSDTH22", (764, 0.05)),
+            ("ETHUSDTZ21", (735, 0.05)),
             ("ETHUSDU20", (401, 0.05)),
             ("ETHUSDU21", (573, 0.05)),
             ("ETHUSDZ20", (430, 0.05)),
@@ -156,9 +167,10 @@ lazy_static! {
             ("FCTM17", (190, 0.000001)),
             ("FCTXBT", (93, 0.000001)),
             ("FILUSDT", (556, 0.01)),
+            ("FTMUSDT", (755, 0.0001)),
             ("GNOM17", (184, 0.000001)),
             ("LINKUSD", (708, 0.001)),
-            ("LINKUSDT", (441, 0.0005)),
+            ("LINKUSDT", (441, 0.001)),
             ("LINKUSDTH21", (465, 0.0005)),
             ("LINKUSDTM21", (528, 0.0005)),
             ("LINKUSDTZ20", (433, 0.0005)),
@@ -179,6 +191,7 @@ lazy_static! {
             ("LTCU20", (395, 0.000005)),
             ("LTCU21", (567, 0.000001)),
             ("LTCUSD", (407, 0.01)),
+            ("LTCUSDT", (738, 0.01)),
             ("LTCXBT", (85, 0.00001)),
             ("LTCZ17", (227, 0.00001)),
             ("LTCZ18", (300, 0.00001)),
@@ -186,14 +199,19 @@ lazy_static! {
             ("LTCZ20", (424, 0.000005)),
             ("LTCZ21", (635, 0.000001)),
             ("LUNAUSD", (649, 0.001)),
+            ("LUNAUSDT", (809, 0.001)),
+            ("MANAUSDT", (781, 0.0001)),
             ("MATICUSDT", (588, 0.0001)),
+            ("METAMEXTUSDT", (803, 0.01)),
             ("NEOG18", (269, 0.000001)),
             ("NEOH18", (270, 0.000001)),
             ("QTUMU17", (195, 0.000001)),
             ("REP7D", (144, 0.000001)),
+            ("SANDUSDT", (780, 0.0001)),
+            ("SHIBUSDT", (754, 0.000000001)),
             ("SNTN17", (202, 0.00000001)),
             ("SOLUSD", (709, 0.01)),
-            ("SOLUSDT", (549, 0.001)),
+            ("SOLUSDT", (549, 0.01)),
             ("SRMUSDT", (632, 0.001)),
             ("SUSHIUSDT", (618, 0.001)),
             ("TRXH19", (325, 0.00000001)),
@@ -237,7 +255,9 @@ lazy_static! {
             ("XBTF15", (1, 0.01)),
             ("XBTF15_G15", (13, 0.01)),
             ("XBTF15_H15", (4, 0.01)),
+            ("XBTF22", (739, 0.5)),
             ("XBTG15", (12, 0.01)),
+            ("XBTG22", (765, 0.5)),
             ("XBTH15", (3, 0.01)),
             ("XBTH15_G15", (14, 0.01)),
             ("XBTH16", (55, 0.01)),
@@ -259,6 +279,7 @@ lazy_static! {
             ("XBTM19", (318, 0.5)),
             ("XBTM20", (361, 0.5)),
             ("XBTM21", (454, 0.5)),
+            ("XBTM22", (758, 0.5)),
             ("XBTN15", (44, 0.01)),
             ("XBTQ15", (46, 0.01)),
             ("XBTU15", (39, 0.01)),
@@ -270,6 +291,9 @@ lazy_static! {
             ("XBTU20", (378, 0.5)),
             ("XBTU21", (532, 0.5)),
             ("XBTUSD", (88, 0.01)),
+            ("XBTUSDT", (732, 0.5)),
+            ("XBTUSDTH22", (763, 0.5)),
+            ("XBTUSDTZ21", (733, 0.5)),
             ("XBTV15", (56, 0.01)),
             ("XBTV21", (650, 0.5)),
             ("XBTX21", (705, 0.5)),
@@ -316,6 +340,7 @@ lazy_static! {
             ("XRPH19", (321, 0.00000001)),
             ("XRPH20", (364, 0.00000001)),
             ("XRPH21", (457, 0.00000001)),
+            ("XRPH22", (760, 0.00000001)),
             ("XRPM17", (189, 0.00000001)),
             ("XRPM18", (275, 0.00000001)),
             ("XRPM19", (329, 0.00000001)),
@@ -327,6 +352,7 @@ lazy_static! {
             ("XRPU20", (396, 0.00000001)),
             ("XRPU21", (568, 0.00000001)),
             ("XRPUSD", (377, 0.0001)),
+            ("XRPUSDT", (736, 0.0001)),
             ("XRPZ17", (228, 0.00000001)),
             ("XRPZ18", (301, 0.00000001)),
             ("XRPZ19", (347, 0.00000001)),
@@ -449,17 +475,22 @@ struct WebsocketMsg<T: Sized> {
     data: Vec<T>,
 }
 
-pub(crate) fn extract_symbol(_market_type: MarketType, msg: &str) -> Option<String> {
-    let ws_msg = serde_json::from_str::<WebsocketMsg<Value>>(msg).unwrap();
+pub(crate) fn extract_symbol(_market_type: MarketType, msg: &str) -> Result<String, SimpleError> {
+    let ws_msg = serde_json::from_str::<WebsocketMsg<Value>>(msg).map_err(|_e| {
+        SimpleError::new(format!(
+            "Failed to deserialize {} to WebsocketMsg<Value>",
+            msg
+        ))
+    })?;
     let symbols = ws_msg
         .data
         .iter()
         .map(|v| v["symbol"].as_str().unwrap())
         .collect::<Vec<&str>>();
     if symbols.is_empty() {
-        None
+        Err(SimpleError::new("data is empty array"))
     } else {
-        Some(symbols[0].to_string())
+        Ok(symbols[0].to_string())
     }
 }
 
@@ -486,8 +517,16 @@ pub(crate) fn get_msg_type(msg: &str) -> MessageType {
     }
 }
 
-pub(crate) fn parse_trade(market_type: MarketType, msg: &str) -> Result<Vec<TradeMsg>> {
-    let ws_msg = serde_json::from_str::<WebsocketMsg<RawTradeMsg>>(msg)?;
+pub(crate) fn parse_trade(
+    market_type: MarketType,
+    msg: &str,
+) -> Result<Vec<TradeMsg>, SimpleError> {
+    let ws_msg = serde_json::from_str::<WebsocketMsg<RawTradeMsg>>(msg).map_err(|_e| {
+        SimpleError::new(format!(
+            "Failed to deserialize {} to WebsocketMsg<RawTradeMsg>",
+            msg
+        ))
+    })?;
     let raw_trades = ws_msg.data;
     let mut trades: Vec<TradeMsg> = raw_trades
         .into_iter()
@@ -529,8 +568,13 @@ pub(crate) fn parse_trade(market_type: MarketType, msg: &str) -> Result<Vec<Trad
 pub(crate) fn parse_funding_rate(
     market_type: MarketType,
     msg: &str,
-) -> Result<Vec<FundingRateMsg>> {
-    let ws_msg = serde_json::from_str::<WebsocketMsg<RawFundingRateMsg>>(msg)?;
+) -> Result<Vec<FundingRateMsg>, SimpleError> {
+    let ws_msg = serde_json::from_str::<WebsocketMsg<RawFundingRateMsg>>(msg).map_err(|_e| {
+        SimpleError::new(format!(
+            "Failed to deserialize {} to WebsocketMsg<RawFundingRateMsg>",
+            msg
+        ))
+    })?;
     let mut rates: Vec<FundingRateMsg> = ws_msg
         .data
         .into_iter()
@@ -584,14 +628,20 @@ pub(crate) fn parse_l2(
     market_type: MarketType,
     msg: &str,
     timestamp: i64,
-) -> Result<Vec<OrderBookMsg>> {
-    let ws_msg = serde_json::from_str::<WebsocketMsg<RawOrder>>(msg)?;
+) -> Result<Vec<OrderBookMsg>, SimpleError> {
+    let ws_msg = serde_json::from_str::<WebsocketMsg<RawOrder>>(msg).map_err(|_e| {
+        SimpleError::new(format!(
+            "Failed to deserialize {} to WebsocketMsg<RawOrder>",
+            msg
+        ))
+    })?;
     let snapshot = ws_msg.action == "partial";
     if ws_msg.data.is_empty() {
         return Ok(Vec::new());
     }
     let symbol = ws_msg.data[0].symbol.clone();
-    let pair = crypto_pair::normalize_pair(&symbol, EXCHANGE_NAME).unwrap();
+    let pair = crypto_pair::normalize_pair(&symbol, EXCHANGE_NAME)
+        .ok_or_else(|| SimpleError::new(format!("Failed to normalize {} from {}", symbol, msg)))?;
     let market_type = if market_type == MarketType::Unknown {
         get_market_type(&symbol, EXCHANGE_NAME, None)
     } else {
