@@ -42,13 +42,8 @@ pub(super) fn calc_quantity_and_volume(
             (quantity_quote / price, quantity_quote, Some(quantity))
         }
         MarketType::LinearSwap | MarketType::LinearFuture | MarketType::Move | MarketType::BVOL => {
-            if exchange == "bitmex" {
-                let quantity_quote = quantity * contract_value;
-                (quantity_quote / price, quantity_quote, Some(quantity))
-            } else {
-                let quantity_base = quantity * contract_value;
-                (quantity_base, quantity_base * price, Some(quantity))
-            }
+            let quantity_base = quantity * contract_value;
+            (quantity_base, quantity_base * price, Some(quantity))
         }
         MarketType::EuropeanOption => {
             let quantity_base = quantity * contract_value;
